import { existsSync } from 'fs';
import { readFile, writeFile } from 'fs/promises';
import { dirname, basename, resolve, posix, sep } from 'path';
import micromatch from 'micromatch';

// ../manifest/dist/generateScriptableBanner.mjs
var SCRIPTABLE_BANNER_STATIC_LINES = [
  "Variables used by Scriptable.",
  "These must be at the very top of the file. Do not edit."
];
var SCRIPTABLE_MANIFEST_KEYS = [
  "name",
  "alwaysRunInApp",
  "shareSheetInputs",
  "iconColor",
  "iconGlyph"
];
function validateManifest(manifest) {
  if (!manifest || typeof manifest !== "object") {
    throw new Error("Manifest must be an object");
  }
  const keys = Object.keys(manifest);
  for (const key of keys) {
    if (!SCRIPTABLE_MANIFEST_KEYS.includes(key)) {
      throw new Error(`Invalid manifest key: ${key}`);
    }
  }
  const typedManifest = manifest;
  if (typedManifest.alwaysRunInApp !== void 0 && typeof typedManifest.alwaysRunInApp !== "boolean") {
    throw new Error("alwaysRunInApp must be a boolean");
  }
  if (typedManifest.name !== void 0 && typeof typedManifest.name !== "string") {
    throw new Error("name must be a string");
  }
  if (typedManifest.iconColor !== void 0 && typeof typedManifest.iconColor !== "string") {
    throw new Error("iconColor must be a string");
  }
  if (typedManifest.iconGlyph !== void 0 && typeof typedManifest.iconGlyph !== "string") {
    throw new Error("iconGlyph must be a string");
  }
  if (typedManifest.shareSheetInputs !== void 0) {
    if (!Array.isArray(typedManifest.shareSheetInputs)) {
      throw new Error("shareSheetInputs must be an array");
    }
    const validInputs = ["file-url", "url", "image", "plain-text"];
    for (const input of typedManifest.shareSheetInputs) {
      if (!validInputs.includes(input)) {
        throw new Error(`Invalid share sheet input: ${input}`);
      }
    }
  }
}
function generateManifestText(manifest = {}, noDefaults = false) {
  manifest && validateManifest(manifest);
  const bannerManifest = {
    "always-run-in-app": manifest.alwaysRunInApp,
    "share-sheet-inputs": manifest.shareSheetInputs?.join(", "),
    "icon-color": manifest.iconColor ?? (noDefaults ? "" : "blue"),
    "icon-glyph": manifest.iconGlyph ?? (noDefaults ? "" : "circle")
  };
  return Object.entries(bannerManifest).filter(([, value]) => value !== void 0 && value !== "").map(([key, value]) => `${key}: ${value};`).join(" ");
}
function generateScriptableBanner(manifest, noDefaults = false) {
  manifest && validateManifest(manifest);
  return [...SCRIPTABLE_BANNER_STATIC_LINES, generateManifestText(manifest, noDefaults)].map((line) => `// ${line}`).join("\n") + "\n";
}

// ../manifest/dist/validateManifest.mjs
var SCRIPTABLE_MANIFEST_KEYS2 = [
  "name",
  "alwaysRunInApp",
  "shareSheetInputs",
  "iconColor",
  "iconGlyph"
];
function validateManifest2(manifest) {
  if (!manifest || typeof manifest !== "object") {
    throw new Error("Manifest must be an object");
  }
  const keys = Object.keys(manifest);
  for (const key of keys) {
    if (!SCRIPTABLE_MANIFEST_KEYS2.includes(key)) {
      throw new Error(`Invalid manifest key: ${key}`);
    }
  }
  const typedManifest = manifest;
  if (typedManifest.alwaysRunInApp !== void 0 && typeof typedManifest.alwaysRunInApp !== "boolean") {
    throw new Error("alwaysRunInApp must be a boolean");
  }
  if (typedManifest.name !== void 0 && typeof typedManifest.name !== "string") {
    throw new Error("name must be a string");
  }
  if (typedManifest.iconColor !== void 0 && typeof typedManifest.iconColor !== "string") {
    throw new Error("iconColor must be a string");
  }
  if (typedManifest.iconGlyph !== void 0 && typeof typedManifest.iconGlyph !== "string") {
    throw new Error("iconGlyph must be a string");
  }
  if (typedManifest.shareSheetInputs !== void 0) {
    if (!Array.isArray(typedManifest.shareSheetInputs)) {
      throw new Error("shareSheetInputs must be an array");
    }
    const validInputs = ["file-url", "url", "image", "plain-text"];
    for (const input of typedManifest.shareSheetInputs) {
      if (!validInputs.includes(input)) {
        throw new Error(`Invalid share sheet input: ${input}`);
      }
    }
  }
}

// src/logger.ts
var levels = {
  verbose: 0,
  info: 1,
  warn: 2,
  error: 3
};
var Logger = class {
  namespace;
  level;
  enabled;
  constructor(namespace, options = {}) {
    this.namespace = namespace;
    this.level = options.level || "info";
    this.enabled = options.enabled ?? true;
  }
  shouldLog(msgLevel) {
    return this.enabled && levels[msgLevel] >= levels[this.level];
  }
  verbose(...args) {
    if (this.shouldLog("verbose")) {
      console.log(`[${this.namespace}]`, ...args);
    }
  }
  info(...args) {
    if (this.shouldLog("info")) {
      console.log(`[${this.namespace}]`, ...args);
    }
  }
  warn(...args) {
    if (this.shouldLog("warn")) {
      console.warn(`[${this.namespace}]`, ...args);
    }
  }
  error(...args) {
    if (this.shouldLog("error")) {
      console.error(`[${this.namespace}]`, ...args);
    }
  }
};
function createLogger(namespace, options) {
  return new Logger(namespace, options);
}
function isMatch(fileName, pattern, options) {
  return micromatch.isMatch(fileName, pattern, options);
}
async function waitForFile(filePath, timeout = 5e3) {
  const start = Date.now();
  while (Date.now() - start < timeout)
    if (existsSync(filePath)) {
      try {
        await readFile(filePath, "utf-8");
        return;
      } catch {
        await new Promise((resolve3) => setTimeout(resolve3, 100));
      }
    }
  throw new Error(`File not available after ${timeout}ms: ${filePath}`);
}
function normalizePath(path) {
  return path.replace(/\\/g, "/");
}

// src/scriptable-banner.ts
var DEFAULT_MANIFEST_EXTENSIONS = [".manifest.json", ".manifest", ".json"];
function scriptableBanner(options = {}) {
  const {
    manifestExtensions = DEFAULT_MANIFEST_EXTENSIONS,
    warnOnMissingManifest = true,
    warnOnInvalidManifest = true,
    warnOnMissingEntry = true,
    getManifestPath,
    patterns = ["**/*.{ts,tsx,js,jsx}"]
  } = options;
  const logger = createLogger("scriptable-banner");
  return {
    name: "scriptable",
    setup(build) {
      let manifestCache = /* @__PURE__ */ new Map();
      if (!build.initialOptions.metafile) {
        logger.warn(
          'Warning: "metafile" option was not enabled. Automatically setting "metafile: true" to allow scriptable plugin to modify output files.'
        );
        build.initialOptions.metafile = true;
      }
      build.onStart(() => {
        manifestCache.clear();
      });
      build.onResolve({ filter: /.*/ }, async (args) => {
        if (!isMatch(args.path, patterns)) {
          return null;
        }
        if (Array.isArray(build.initialOptions.entryPoints) && build.initialOptions.entryPoints.map(String).includes(args.path)) {
          const dir = dirname(args.path);
          const baseFileName = basename(args.path).replace(/\.[tj]sx?$/, "");
          const manifestPath = getManifestPath ? getManifestPath(args.path) : manifestExtensions.map((ext) => resolve(dir, `${baseFileName}${ext}`)).find(existsSync);
          if (manifestPath) {
            try {
              const manifestContent = await readFile(manifestPath, "utf8");
              const manifest = JSON.parse(manifestContent);
              const normalizedPath = normalizePath(args.path);
              try {
                validateManifest2(manifest);
              } catch (error) {
                if (warnOnInvalidManifest && error instanceof Error) {
                  logger.warn(`Invalid manifest file: ${manifestPath} - ${error.message}`, error);
                }
                return null;
              }
              manifestCache.set(normalizedPath, manifest);
            } catch (error) {
              logger.warn(`Failed to read manifest file: ${manifestPath}`, error);
            }
          } else if (warnOnMissingManifest) {
            logger.warn(`No manifest file found for entry point: ${args.path}`);
          }
        }
        return null;
      });
      build.onEnd(async (result) => {
        if (!result.metafile || Object.keys(result.metafile.outputs).length === 0) {
          logger.warn(
            'Skipping banner addition because "metafile" is not available or contains no outputs. Ensure "metafile: true" is set in esbuild options if write is enabled.'
          );
          return;
        }
        const outputs = result.metafile.outputs;
        for (const outputPath of Object.keys(outputs)) {
          const outputMeta = outputs[outputPath];
          const entryPoint = outputMeta?.entryPoint ?? "";
          const normalizedEntryPath = entryPoint ? posix.normalize(entryPoint.split(sep).join("/")) : "";
          if (!normalizedEntryPath || !manifestCache.has(normalizedEntryPath)) {
            if (warnOnMissingEntry) {
              logger.warn(`No matching entry point found for output file: ${outputPath}`);
            }
            continue;
          }
          try {
            const manifest = manifestCache.get(normalizedEntryPath) || {};
            const banner = generateScriptableBanner(manifest);
            if (result.outputFiles) {
              const outputFile = result.outputFiles.find((file) => file.path === outputPath || file.path === "<stdout>");
              if (outputFile) {
                outputFile.contents = Buffer.from(banner + outputFile.text.trimEnd());
              }
            } else {
              try {
                await waitForFile(outputPath);
                const content = await readFile(outputPath, "utf-8");
                const updatedContent = banner + content.trimEnd();
                await writeFile(outputPath, updatedContent, "utf8");
              } catch (error) {
                logger.error(
                  `Failed to process file ${outputPath}: ${error instanceof Error ? error.message : String(error)}`
                );
              }
            }
          } catch (error) {
            logger.error(`Failed to process output file: ${outputPath}`, error);
          }
        }
      });
    }
  };
}

export { scriptableBanner };
