import {existsSync} from 'fs';
import {readFile} from 'fs/promises';
import micromatch from 'micromatch';
import {resolve} from 'path';

/**
 * Creates a file pattern matcher function
 * @param patterns - Glob patterns to match files
 * @returns A function that returns true if the file matches the patterns
 */
export function createPatternMatcher(patterns?: string | string[]) {
  if (!patterns) return () => true;

  const patternArray = Array.isArray(patterns) ? patterns : [patterns];
  const includePatterns = patternArray.filter(p => !p.startsWith('!'));
  const excludePatterns = patternArray.filter(p => p.startsWith('!')).map(p => p.slice(1));

  return (fileName: string): boolean => {
    return micromatch.isMatch(fileName, includePatterns, {
      dot: true,
      ignore: excludePatterns,
    });
  };
}

/**
 * Checks if a file name matches a pattern
 *
 * @param fileName - The file name to check
 * @param pattern - The pattern to match
 * @param options - The options to pass to micromatch
 * @returns True if the file name matches the pattern
 */
export function isMatch(fileName: string, pattern: string | readonly string[], options?: micromatch.Options): boolean {
  return micromatch.isMatch(fileName, pattern, options);
}

// Delayed file check function
export async function waitForFile(filePath: string, timeout = 5000): Promise<void> {
  const start = Date.now();
  while (Date.now() - start < timeout)
    if (existsSync(filePath)) {
      try {
        await readFile(filePath, 'utf-8'); // Check if file is readable

        return; // If the file is readable, exit the loop
      } catch {
        await new Promise(resolve => setTimeout(resolve, 100)); // Try again after a delay
      }
    }
  throw new Error(`File not available after ${timeout}ms: ${filePath}`);
}

export function normalizePath(path: string): string {
  return path.replace(/\\/g, '/');
}

export const np = normalizePath;

export function isPathEqual(path1: string, path2: string): boolean {
  return resolve(path1) === resolve(path2);
}
